#! /usr/bin/perl

## Variable Declartion
my (%reportData, @reportKeys);
my (%Vol,@reportKeys);
my @volKeys;
my $inode_Used,$inode_Total,$inode_Available ;

## The varialbles starting with the name inode are used to set the comment field names

my $inode_Used="Inode Used";
my $inode_Total="Inode Total";
my $inode_Available="Inode Available";

## The variables starting with report are used to create the custom report

my $report_Catalog_Name="Volume";
my $report_Long_Name="Volume Inode Usage";
my $report_Display_Tab="Filesystems";
my $report_Description="This is a custom created report to get the inode used and available in each volume."; 
my $report_User_Name="inodeUsage";
my $report_Fields="Volume\.fullname\=\"Vol Full Name\"\,Volume\.inodesusedpct\,\Volume\.comment"; 
my $dfm_install_dir = get_dfmdir();

## The variables used for thresholds

my $inodeWarningThreshold;
my $inodeErrorThreshold;

## Read threshold values from file

open(FH, "$dfm_install_dir\/log\/inodeThreshold.txt") ||  printErrorAndExit("\nNot able to open file inodeThreshold.txt");

my $temp = <FH>;
chomp($temp);
my($temp, $inodeWarningThreshold) = split(/=/,$temp);

my $temp = <FH>;
chomp($temp);
my($temp, $inodeErrorThreshold) = split(/=/,$temp);

print "\n\nWarn=$inodeWarningThreshold and Error=$inodeErrorThreshold\n\n";

## Read the options 
if( $ARGV[0] =~ m[^cleanup]i )
{
	cleanup();
}
	
 
## Preconfiguration Script Creates comment fields and Custom Report
if( !-e "$dfm_install_dir/log/Inode_Rep_Status.txt")
{
	my $out = `dfm comment field list`;
 	if( $out =~ /$inode_Used\s+/ )
	{
		printErrorAndExit("$inode_Used comment field");
	}
	else
	{
		executeDFMCmd("dfm comment field create \"$inode_Used\"");
	}

	if( $out =~ /$inode_Total\s+/ )
	{
		 printErrorAndExit("$inode_Total  comment field");
	}
	else
	{
		executeDFMCmd("dfm comment field create \"$inode_Total\"");
	}		
	if( $out =~ /$inode_Available\s+/ )
	{
		printErrorAndExit("$inode_Available comment field");

	}
	else
        {
		executeDFMCmd("dfm comment field create \"$inode_Available\"");
	}

	$out = `dfm report list -t custom`;
	if( $out =~ /$report_User_Name\s+/ )
	{
		printErrorAndExit("$report_User_Name Report");
		
	}
	else
	{
		executeDFMCmd("dfm report create -R $report_Catalog_Name -f $report_Fields -L \"$report_Long_Name\" -d \"$report_Description\" -D $report_Display_Tab  $report_User_Name");
	}
	open(FH, ">$dfm_install_dir/log/Inode_Rep_Status.txt");
	close(FH);
}

## Run the volumes Report and get the volumes
my $volume_report = `dfm report view -F perl volumes `;
eval($volume_report);
%Vol = %reportData;
@volKeys = @reportKeys;


## Main Code that calculates the inode used percentage for a volume
## Key = Volume Id
foreach my $key1 (@volKeys)
{
                        
                  my $output = `dfm detail -F perl $key1`;
                  eval($output);
			my $inode_U = $details{$key1}{'dfInodesUsed'};
			my $inode_T = $details{$key1}{'dfInodesTotal'}; 
			my $inode_A = ($inode_T - $inode_U);
			my $inode_UP = $details{$key1}{'dfInodesPercent'};;
 
			executeDFMCmd("dfm comment set $key1 \"$inode_Used=$inode_U\"");	
			executeDFMCmd("dfm comment set $key1 \"$inode_Total=$inode_T \"");
			executeDFMCmd("dfm comment set $key1 \"$inode_Available=$inode_A\"");
			eventGen($key1, $inode_UP);
}	


## Generate event, if threshold breached

sub eventGen
{
	my($volName, $inodeUP) = @_;
	
	if( $inodeUP >= $inodeWarningThreshold && $inodeUP < $inodeErrorThreshold )
	{
		my $output  = `dfm event generate "inodeUsage:inode-utilization:AlmostFull" $volName`;
	}
	elsif ( $inodeUP >= $inodeErrorThreshold )
	{
		my $output  = `dfm event generate "inodeUsage:inode-utilization:Full" $volName`;
	}
}
	
	

## Execute DFM command
sub executeDFMCmd
{
	my($cmd) = @_;
	my $output = `$cmd`;
	if( $output =~ /Error/i )
	{
		print "\nConfiguration Script Failed!!!!!!\n";
		print "\nReason: $output";
	}
	else
	{
		print "\n$output";
	}
}


## Returns the dfm installed directory
sub get_dfmdir
{
my ($status, $output, @lines, $dfmdir);

        $output = `dfm about`;
        @lines = split "\n", $output;

        foreach (@lines)
        {
                if (m[^Installation Directory\s+(.*)])
                {
                        $dfmdir = $1;
                }
        }

        if (!$dfmdir)
        {
                die "Can't find install directory\n";
        }

        if (!-d $dfmdir)
        {
                die "Parsed install directory '$dfmdir' doesn't exist\n";
        }

        return $dfmdir;
}

## Print the Appropriate Error and exit the script
sub printErrorAndExit
{
	my($name) = @_;
	print "\nERROR: $name already exists. Rename/Delete the existing field\n";
	exit 0;
}	


## Cleanup the comment fields, Custom report and log file
sub cleanup
{
	my $out;
	
	$out = `dfm comment field destroy \"$inode_Available\"`;
	print "$out\n";
	$out = `dfm comment field destroy \"$inode_Total\"`;
	print "$out\n";	
	$out = `dfm comment field destroy \"$inode_Used\"`;
	print "$out\n";
	$out = `dfm report destroy $report_User_Name`;
	print "$out\n";
	print "\n$dfm_install_dir\/logs\/Inode_Rep_Status.txt\n";
	$out = unlink("$dfm_install_dir\/log\/Inode_Rep_Status.txt");
	print "\n$out\n";
	exit;
}
		
			
